unit UltraFavTree;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  ActiveX,
  MPShellUtilities,
  MPCommonUtilities,
  MPCommonObjects,
  MPDataObject,
  MPShellTypes,
  ShellAPI,
  ShlObj,
  TntActnList,
  TntSysUtils,
  TntClasses,
  TntRegistry;

const
  STR_GLOBALFAVSETTINGS_FILE = 'UltraFavTree.cfg';

type
  IUltraMenuCallback = interface(IUnknown)
  ['{05E36653-F749-4952-8582-79042EA50433}']
    procedure BrowseToByCallback(PIDL: PItemIDList; NewTab: Boolean);
  end;


  TUltraLinkManager = class
  private
    FFileLinks: TTntStringList;
    FFolderLinks: TTntStringList;
    function GetHasFileLinks: Boolean;
    function GetIsNTFS(Drive: WideChar): Boolean;
    function GetIsNTFS_5(Drive: WideChar): Boolean;
    function GetLinkFolderLinks: Boolean;
  protected
    procedure ClearLinkTargets;
  public
    constructor Create;
    destructor Destroy; override;
    procedure StoreLinkTargets(const DataObject: IDataObject);
    property HasFileLinks: Boolean read GetHasFileLinks;
    property HasFolderLinks: Boolean read GetLinkFolderLinks;
    property IsNTFS[Drive: WideChar]: Boolean read GetIsNTFS;
    property IsNTFS_5[Drive: WideChar]: Boolean read GetIsNTFS_5;
    property FileLinks: TTntStringList read FFileLinks write FFileLinks;
    property FolderLinks: TTntStringList read FFolderLinks write FFolderLinks;
  end;


  TUltraFavoritesItem = class
  private
    FCaption: WideString;
    FPIDL: PItemIDList;
    FSubItemList: TList;
    function GetHasSubItems: Boolean;
    function GetSubItemCount: Integer;
    function GetSubItems(Index: Integer): TUltraFavoritesItem;
    procedure SetSubItems(Index: Integer; Value: TUltraFavoritesItem);
  protected
    procedure Read(S: TStream);
    procedure Write(S: TStream);
    property SubItemList: TList read FSubItemList write FSubItemList;
  public
    constructor Create;
    destructor Destroy; override;
    function AddChild(ACaption: WideString; APIDL: PItemIDList): TUltraFavoritesItem;
    procedure ClearSubItems;
    procedure DeleteChild(Child: TUltraFavoritesItem);
    property Caption: WideString read FCaption write FCaption;
    property HasSubItems: Boolean read GetHasSubItems;
    property PIDL: PItemIDList read FPIDL write FPIDL;
    property SubItemCount: Integer read GetSubItemCount;
    property SubItems[Index: Integer]: TUltraFavoritesItem read GetSubItems write SetSubItems;
  end;

  TUltraFavoritesItemRoot = class(TUltraFavoritesItem)
  public
    procedure BuildMenu(ParentMenu: HMENU);
    procedure LoadFromFile(FileName: WideString; var StreamVersion: Integer);
    procedure LoadFromStream(S: TStream; var StreamVersion: Integer);
    procedure SaveToFile(FileName: WideString; StreamVersion: Integer);
    procedure SaveToStream(S: TStream; StreamVersion: Integer);
  end;

  function AppFilesPath: WideString;
  function ApplicationPath: WideString;
  function GlobalFavsSettingsFile: WideString;

implementation

function ApplicationPath: WideString;
var
  Reg: TTntRegistry;
begin
  Reg := TTntRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKeyReadOnly('\Software\Mustangpeak\UltraExplorer') then
    begin
      Result := Reg.ReadString('Path');
    end else
      Result := ModuleFileName(True)    // Backup if key does not exist, shell extensions won't work if we hit this.
  finally
    Reg.Free
  end
end;

function ProgramFilesFolderPath: WideString;
var
  P: PWideChar;
begin
  if Assigned(SHGetKnownFolderPath_MP) then
  begin
    if Succeeded(SHGetKnownFolderPath_MP(FOLDERID_ProgramFiles, KF_FLAG_NO_ALIAS, 0, P)) then
    begin
      Result := P;
      PIDLMgr.FreeOLEStr(P);
    end else
      Result := ProgramFilesFolder.NameForParsing;
  end else
    Result := ProgramFilesFolder.NameForParsing
end;

function AppFilesPath: WideString;
begin
  Result := '';
  {$IFDEF PORTABLE}
  Result := ApplicationPath
  {$ELSE}
  if IsWinVista {and (Pos(ProgramFilesFolderPath, ApplicationPath) > 0)} then
  begin
    if not WideDirectoryExists(UserDocumentsFolder.NameForParsing + '\' + 'UltraExplorer') then
      WideCreateDir(UserDocumentsFolder.NameForParsing + '\' + 'UltraExplorer');
    Result := UserDocumentsFolder.NameForParsing + '\' + 'UltraExplorer';
  end else
    Result := ApplicationPath
  {$ENDIF}
end;

function GlobalFavsSettingsFile: WideString;
begin
  Result := AppFilesPath + '\' + STR_GLOBALFAVSETTINGS_FILE
end;

{ TUltraFavoritesItem }
constructor TUltraFavoritesItem.Create;
begin
  inherited Create;
  SubItemList := TList.Create;
end;

destructor TUltraFavoritesItem.Destroy;
begin
  ClearSubItems;
  FreeAndNil(FSubItemList);
  PIDLMgr.FreeAndNilPIDL(FPIDL);
  inherited Destroy;
end;

function TUltraFavoritesItem.AddChild(ACaption: WideString; APIDL: PItemIDList): TUltraFavoritesItem;
begin
  Result := TUltraFavoritesItem.Create;
  Result.PIDL := PIDLMgr.CopyPIDL(APIDL);
  Result.Caption := ACaption;
  SubItemList.Add(Result);
end;

function TUltraFavoritesItem.GetHasSubItems: Boolean;
begin
  Result := FSubItemList.Count > 0
end;

function TUltraFavoritesItem.GetSubItemCount: Integer;
begin
  Result := FSubItemList.Count;
end;

function TUltraFavoritesItem.GetSubItems(Index: Integer): TUltraFavoritesItem;
begin
  Result := TUltraFavoritesItem( SubItemList[Index])
end;

procedure TUltraFavoritesItem.ClearSubItems;
var
  i: Integer;
begin
  try
    for i := 0 to SubItemCount - 1 do
      TObject( SubItemList[i]).Free;
  finally
    SubItemList.Count := 0
  end
end;

procedure TUltraFavoritesItem.DeleteChild(Child: TUltraFavoritesItem);
var
  i: Integer;
begin
  i := SubItemList.IndexOf(Child);
  if i > -1 then
  begin
    TObject( SubItemList[i]).Free;
    SubItemList.Delete(i)
  end
end;

procedure TUltraFavoritesItem.Read(S: TStream);
var
  i, Len: Integer;
begin
  Caption := StreamHelper.ReadWideString(S);
  PIDL := PIDLMgr.LoadFromStream(S);
  Len := StreamHelper.ReadInteger(S);
  for i := 0 to Len - 1 do
  begin
    AddChild('', nil);
    TUltraFavoritesItem( SubItemList[i]).Read(S)
  end
end;

procedure TUltraFavoritesItem.SetSubItems(Index: Integer; Value: TUltraFavoritesItem);
begin
  SubItemList[Index] := Value
end;

procedure TUltraFavoritesItem.Write(S: TStream);
var
  i: Integer;
begin
  StreamHelper.WriteWideString(S, Caption);
  PIDLMgr.SaveToStream(S, PIDL);
  StreamHelper.WriteInteger(S, SubItemCount);
  for i := 0 to SubItemCount - 1 do
    TUltraFavoritesItem( SubItemList[i]).Write(S)
end;

{ TUltraFavoritesItemRoot}
procedure TUltraFavoritesItemRoot.BuildMenu(ParentMenu: HMENU);
begin

end;

procedure TUltraFavoritesItemRoot.LoadFromFile(FileName: WideString;
  var StreamVersion: Integer);
var
  F: TTntFileStream;
begin
  F := TTntFileStream.Create(FileName, fmOpenRead or fmShareExclusive);
  try
    LoadFromStream(F, StreamVersion)
  finally
    F.Free
  end
end;

procedure TUltraFavoritesItemRoot.LoadFromStream(S: TStream;
  var StreamVersion: Integer);
begin
  ClearSubItems;
  StreamVersion := StreamHelper.ReadInteger(S);
  Read(S)
end;

procedure TUltraFavoritesItemRoot.SaveToFile(FileName: WideString;
  StreamVersion: Integer);
var
  F: TTntFileStream;
begin
  F := TTntFileStream.Create(FileName, fmCreate	or fmShareExclusive);
  try
    SaveToStream(F, StreamVersion)
  finally
    F.Free
  end
end;

procedure TUltraFavoritesItemRoot.SaveToStream(S: TStream;
  StreamVersion: Integer);
begin
  StreamHelper.WriteInteger(S, StreamVersion);
  Write(S)
end;

{ TUltraLinkManager }
constructor TUltraLinkManager.Create;
begin
  inherited Create;
  FolderLinks := TTntStringList.Create;
  FileLinks := TTntStringList.Create;
end;

destructor TUltraLinkManager.Destroy;
begin
  FolderLinks.Free;
  FileLinks.Free;
  inherited Destroy;
end;

function TUltraLinkManager.GetHasFileLinks: Boolean;
begin
  Result := FileLinks.Count > 0
end;

function TUltraLinkManager.GetIsNTFS(Drive: WideChar): Boolean;
var
  Name, SysName: array[0..MAX_PATH] of Char;
  S: WideString;
  Serial, MaxComponentLen, Flags: DWORD;
begin
  Result := False;
  if Drive = '' then
     Exit;
  if GetVolumeInformation(PChar( Drive + ':\'), Name, MAX_PATH, @Serial, MaxComponentLen, Flags, SysName, MAX_PATH) then
  begin
    S := SysName;
    Result := 'NTFS' = S
  end
end;

function TUltraLinkManager.GetIsNTFS_5(Drive: WideChar): Boolean;
begin
  Result := IsNTFS[Drive] and IsWinVista
end;

function TUltraLinkManager.GetLinkFolderLinks: Boolean;
begin
  Result := FolderLinks.Count > 0
end;

procedure TUltraLinkManager.ClearLinkTargets;
begin
  FolderLinks.Clear;
  FileLinks.Clear;
end;

procedure TUltraLinkManager.StoreLinkTargets(const DataObject: IDataObject);
var
  HDrop: TCommonHDrop;
  i: Integer;
begin
  ClearLinkTargets;
  HDrop := TCommonHDrop.Create;
  try
    HDrop.LoadFromDataObject(DataObject);
    for i := 0 to HDrop.FileCount - 1 do
    begin
      if WideDirectoryExists(HDrop.FileName(i)) then
        FolderLinks.Add(HDrop.FileName(i))
      else
      if WideFileExists(HDrop.FileName(i)) then
        FileLinks.Add(HDrop.FileName(i))
    end
  finally
    HDrop.Free
  end
end;

end.
