unit UnitTaskPlugins;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  ShellAPI,
  ShlObj,
  Forms,
  MPCommonObjects,
  UltraUserStrings,
  UltraTaskInf,
  TntSysUtils,
  TntClasses,
  MPCommonUtilities,
  TntWindows;

const
  STR_ULTRAPLUGINSERVERFUNCTION: PChar = 'CreatePluginServer'; // Name of exported function
  STR_ULTRAPLUGINVCLFUNCTION: PChar = 'SetVCLVariables';

  MAX_PLUGIN_STR_LEN = 128;  // Captions can only be 128 characters long

type
    //
  // Hold information about a Task Library DLL
  //
  TUltraTaskLib = class
  private
    FFileName: WideString;
    FLibHandle: THandle;
    FPlugin: IUltraPlugin;
    function GetPlugin: IUltraPlugin;
    function GetPluginCount: Integer;
  public
    property FileName: WideString read FFileName write FFileName;
    property LibHandle: THandle read FLibHandle write FLibHandle;
    property Plugin: IUltraPlugin read GetPlugin;
    property PluginCount: Integer read GetPluginCount;
  end;

  TUltraTaskPlugins = class
  private
    FTaskLibList: TList;
    function GetCaption(TaskIndex, PluginIndex: Integer): WideString;
    function GetCount: Integer;
    function GetTask(Index: Integer): TUltraTaskLib;
  protected
    property TaskLibList: TList read FTaskLibList write FTaskLibList;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    function LibraryExists(LibPath: WideString): Boolean;
    procedure Load;
    procedure Release;
    property Caption[TaskIndex, PluginIndex: Integer]: WideString read GetCaption;
    property Count: Integer read GetCount;
    property Task[Index: Integer]: TUltraTaskLib read GetTask; default;
  end;

implementation

{ TUltraTaskPlugins }
constructor TUltraTaskPlugins.Create;
begin
  inherited Create;
  TaskLibList := TList.Create;
end;

destructor TUltraTaskPlugins.Destroy;
begin
  Release;
  FreeAndNil(FTaskLibList);
  inherited Destroy;
end;

function TUltraTaskPlugins.GetCaption(TaskIndex, PluginIndex: Integer): WideString;
var
  Buffer: array[0..MAX_PLUGIN_STR_LEN] of WideChar;
begin
  Result := '';
  if Assigned(Task[TaskIndex].Plugin) then
  begin
    FillChar(Buffer, SizeOf(Buffer), #0);
    if Task[TaskIndex].Plugin.GetPluginName(PluginIndex, MAX_PLUGIN_STR_LEN, @Buffer) then
      Result := Buffer;
  end
end;

function TUltraTaskPlugins.GetCount: Integer;
begin
  Result := TaskLibList.Count
end;


function TUltraTaskPlugins.GetTask(Index: Integer): TUltraTaskLib;
begin
  Result := TUltraTaskLib( TaskLibList[Index])
end;

function TUltraTaskPlugins.LibraryExists(LibPath: WideString): Boolean;
var
  i: Integer;
begin
  Result := False;
  i := 0;
  while not Result and (i < TaskLibList.Count) do
  begin
    Result := WideCompareStr(TUltraTaskLib( TaskLibList[i]).FileName, LibPath) = 0;
    Inc(i);
  end;
end;

procedure TUltraTaskPlugins.Load;

  procedure AddLibrary(FileName: WideString);
  var
    TaskLibrary: TUltraTaskLib;
    Lib: THandle;
    SetVCLVariables: TSetVCLVariablesFunc;
  begin
    Lib := Tnt_LoadLibraryW(PWideChar( FileName));
    if Lib <> 0 then
    begin
      if (GetProcAddress(Lib, STR_ULTRAPLUGINSERVERFUNCTION) <> nil) and
         (GetProcAddress(Lib, STR_ULTRAPLUGINVCLFUNCTION) <> nil) then
      begin
        TaskLibrary := TUltraTaskLib.Create;
        TaskLibrary.FileName := FileName;
        TaskLibrary.LibHandle := Lib;
        TaskLibList.Add(TaskLibrary);
        SetVCLVariables := GetProcAddress(Lib, STR_ULTRAPLUGINVCLFUNCTION);
        SetVCLVariables(Longword( Application), Longword( Screen), Longword( HintWindowClass)); 
      end
    end
  end;

var
  Data: TWin32FindDataW;
  FindHandle: THandle;
  Path, LibPath: WideString;
begin
  Path := ModuleFileName + '\' + STR_TASKLIB_FOLDER;
  FindHandle := Tnt_FindFirstFileW(PWideChar( Path + '\*' + STR_TASKLIB_EXT), Data) ;
  if FindHandle <> INVALID_HANDLE_VALUE then
  begin
    LibPath := Path + '\' + Data.cFileName;
    if not LibraryExists(LibPath) then
      AddLibrary(LibPath);
    while Tnt_FindNextFileW(FindHandle, Data) do
    begin
      LibPath := Path + '\' + Data.cFileName;
      if not LibraryExists(LibPath) then
        AddLibrary(LibPath);
    end;
    Windows.FindClose(FindHandle)
  end;
end;

procedure TUltraTaskPlugins.Release;
var
  i: Integer;
begin
  try
    for i := 0 to TaskLibList.Count - 1 do
    begin
      Task[i].FPlugin := nil;
      FreeLibrary(Task[i].LibHandle);
      TObject( TaskLibList[i]).Free;
    end
  finally
    TaskLibList.Count := 0
  end
end;

{ TUltraTaskLib }
function TUltraTaskLib.GetPlugin: IUltraPlugin;
var
  Func: TCreatePluginServerFunc;
begin
  if not Assigned(FPlugin) then
  begin
    Func := GetProcAddress(LibHandle, STR_ULTRAPLUGINSERVERFUNCTION);
    if Assigned(Func) then
      FPlugin := Func
  end;
  Result := FPlugin;
end;

function TUltraTaskLib.GetPluginCount: Integer;
begin
  Result := 0;
  if Assigned(Plugin) then
    if not Plugin.GetPluginCount(Result) then
      Result := 0;
end;

end.
